function enh=WienerSubtraction(ns,fs,kind,IS)
%
% Wiener or subtraction filtering for noise reduction
%
% Input Parameters :  
%   ns          noisy speech 
%   fs          frequency sampling (in Hz)
%   IS          initial noise duration (in number of samples)
%   kind        filter kind: 'w' - Wiener; 's' - subtraction
%
% Output Parameters :
%   rest        enhanced speech

% USAGE:    rest=WienerSubtraction(ns,fs,IS,'w')  %Wiener filtering
%           rest=WienerSubtraction(ns,fs,IS,'s')  %spectral subtraction filtering

%Author :       LIU Ming, 2008
%Modified :     SCALART Pascal October, 2008
%Modified :     PRODEUS Arkadiy March-April, 2015
%
%% ------- input noisy speech  --------
l = length(ns);
s=ns;
wt = 0.032;                         % window length in sec
wl = fix(wt*fs)+mod(fix(wt*fs),2);  % window length im samples (must be even)
NFFT=2*wl;                          % FFT size is twice the window length
hanwin = hanning(wl);               % hanning window

if (nargin<4 || isstruct(IS))
    IS=10*wl;                       % ten frames of initial noise
end
%% -------- compute noise power spectrum ----------
nsum = zeros(NFFT,1);

count = 0; 
    for m = 0:IS-wl
     nwin = s(m+1:m+wl).*hanwin;	
      nsum = nsum + abs(fft(nwin,NFFT)).^2;
     count = count + 1;
    end

d= (nsum)/count;                  % noise power spectrum estimation
%% --------- main algorithm ---------------
SP = 0.5;                     % for Tinc = 0.016 sec
Tinc = wt*SP;                 % Tinc = 0.016 sec
normFactor=1/SP;              % normalization factor
overlap = fix((1-SP)*wl);     % overlap between sucessive frames
offset = wl - overlap;        % frame shift in samples
max_m = fix((l-NFFT)/offset); 

oldmag = zeros(NFFT,1);
news = zeros(l,1);            % zone for enhanced signal

alpha = exp(-Tinc/0.396); %     averaging with RC=0.396 (by Ephraim-Malah)
%% --------------- Iteration to remove noise ---------------------
for m = 0:max_m
   begin = m*offset+1;    
   iend = m*offset+wl;
   speech = ns(begin:iend);       %extract speech segment
   winy = hanwin.*speech;         %perform hanning window
   ffty = fft(winy,NFFT);         %perform fast fourier transform
   phasey = angle(ffty);          %extract phase
   magy = abs(ffty);              %extract magnitude
   raprsnr = ((magy.^2) ./ d)-1 ; %calculate rough a priori SNR
   raprsnr=max(raprsnr,0);        %must be non-negative
      
   % calculate filter Frequency Response
   if kind == 'w'
       % calculate a priori SNR using decision directed approach
       xi = alpha * ( (oldmag.^2)./d ) + (1-alpha) * raprsnr;
       FreqResp = xi./(xi+1);
   else
       FreqResp = sqrt(raprsnr./(raprsnr+1));
   end
   newmag = FreqResp.*  magy;
   oldmag = newmag;
   ffty = newmag.*exp(1i*phasey);
   news(begin:begin+NFFT-1) = news(begin:begin+NFFT-1) + real(ifft(ffty,NFFT))/normFactor;
   
end
enh=news;            % enhanced signal
